import pyperclip
from selenium.webdriver import Keys
from behave import *
from api.A import A
from volworld_aws_api_common.test.behave.selenium_utils import w__get_element_by_shown_dom_id, \
    w__click_element_by_dom_id, \
    click_element, scroll_to_bottom, w__assert_element_existing, w__assert_element_not_existing
from volworld_aws_api_common.test.behave.row_utils import assert_tag_icon_class_of_list
from selenium.webdriver.common.by import By
from volworld_common.test.behave.BehaveUtil import BehaveUtil

from src.volworld_word_book_library_se_test.nav_utils import open_nav_more_actions_drawer
from test.behave.CotA import CotA
from test.behave.PageGroup import PageGroup


def check_word_count_in_32e_page(c, words):
    if BehaveUtil.clear_int(words) == 0:
        empty_list_container = w__get_element_by_shown_dom_id(c, [A.Empty, A.InfoRow, A.List])
        assert empty_list_container is not None
        return

    list_container = w__get_element_by_shown_dom_id(c, [A.InfoRow, A.List])
    word_rows = list_container.find_elements(By.XPATH, "./div")
    assert len(word_rows) == BehaveUtil.clear_int(words)


def check_unknown_word_count_in_32e_page(c, words):
    list_container = w__get_element_by_shown_dom_id(c, [A.InfoRow, A.List])
    btns = list_container.find_elements(By.XPATH, "./div/main/aside/button[1]")
    class_name = f"SvgIcon-{'-'.join([A.Not, A.Find])}"
    unknown_btn_count = 0
    for btn in btns:
        if btn.get_attribute('innerHTML').strip().find(class_name) > -1:
            unknown_btn_count += 1
    assert unknown_btn_count == BehaveUtil.clear_int(words)


@then('there are "{words}" words in word list')
def then_words_in_word_list(c, words: str):
    target_page = getattr(c, CotA.TargetPage)
    if PageGroup.is_group(c, PageGroup.Pg32E_Chapter_Word_List_Editor_Page):
        check_word_count_in_32e_page(c, words)
    else:
        assert False, f"Not support page = [{target_page}]"


@then('there are "{words}" unknown words in word list')
def then_unknown_words_in_word_list(c, words: str):
    target_page = getattr(c, CotA.TargetPage)

    if PageGroup.is_group(c, PageGroup.Pg32E_Chapter_Word_List_Editor_Page):
        check_unknown_word_count_in_32e_page(c, words)
    else:
        assert False, f"Not support page = [{target_page}]"


@then('[save button] in [bottom app bar] is "{active}"')
def check_save_chapter_btn(c, active: str):
    btn = w__get_element_by_shown_dom_id(c, [A.BottomAppBar, A.Save, A.Button])
    assert btn is not None
    enabled = BehaveUtil.clear_string(active).lower() == 'enabled'
    assert btn.is_enabled() == enabled


def load_word_text_list(c) -> list:
    elm = w__get_element_by_shown_dom_id(c, [A.InfoRow, A.List])
    assert elm is not None
    rows = elm.find_elements(By.XPATH, "./div/main/main/span")
    row_text_list = []
    for r in rows:
        row_text_list.append(r.get_attribute('innerHTML').strip())
        # print(r.get_attribute('innerHTML').strip())
    return row_text_list


def load_drawer_tag_info_list(c) -> list:
    elm = w__get_element_by_shown_dom_id(c, [A.InfoRow, A.List])
    assert elm is not None
    rows = elm.find_elements(By.XPATH, "./div/main/nav/main/b")
    row_text_list = []
    for r in rows:
        row_text_list.append(r.get_attribute('innerHTML').strip())
        print(r.get_attribute('innerHTML').strip())
    return row_text_list


@then('the word list is sort by {sort_type} in {dir_type} order')
def then_the_word_list_is_sort_by(c, sort_type: str, dir_type: str):
    dir_type = BehaveUtil.clear_string(dir_type).lower()
    sort_type = BehaveUtil.clear_string(sort_type).lower()
    row_bool_title_list = load_word_text_list(c)
    sort_text = row_bool_title_list.copy()
    row_tag_list = load_drawer_tag_info_list(c)
    row_tag_list = list(map(int, row_tag_list))
    sort_tag = row_tag_list.copy()
    assert_tag = True
    if sort_type == 'title' or sort_type == 'alphabetical':
        sort_text.sort()
        assert_tag = False
    else:
        sort_tag.sort()

    if dir_type == 'descending':
        sort_text.reverse()
        sort_tag.reverse()

    if assert_tag:
        for i in range(len(sort_tag)):
            assert sort_tag[i] == row_tag_list[i], f"ori = [{row_tag_list}]\nsort = [{sort_tag}]"
    else:
        for i in range(len(sort_text)):
            assert sort_text[i] == row_bool_title_list[i], f"ori = [{row_bool_title_list}]\nsort = [{sort_text}]"


def click_on_word_row_button(c, word: str, ind: int):
    list_container = w__get_element_by_shown_dom_id(c, [A.InfoRow, A.List])
    spans = list_container.find_elements(By.XPATH, f'./div/main/main/span[text()=\'{BehaveUtil.clear_string(word)}\']')
    assert len(spans) == 1, f'spans = {len(spans)}'
    scroll_to_bottom(c)
    btn = spans[0].find_element(By.XPATH, f'../../aside/button[{ind}]')
    assert btn is not None
    # print(btn.get_attribute('innerHTML'))
    click_element(c, btn)


@when('"{mentor}" click on [voice icon] of word "{word}" in [32E_Chapter-Word-List-Editor-Page]')
def click_unknown_word(c, mentor: str, word: str):
    click_on_word_row_button(c, word, 1)


@then('[save chapter button] in [bottom app bar] is shown')
def check_save_chapter_btn_not_show(c):
    w__assert_element_existing(c, [A.BottomAppBar, A.Save, A.Button])


@then('[save chapter button] in [bottom app bar] is NOT shown')
def check_save_chapter_btn_not_show(c):
    w__assert_element_not_existing(c, [A.BottomAppBar, A.Save, A.Button])


def is_word_in_used_word_list(c, text: str):
    if PageGroup.is_group(c, PageGroup.Pg32E_Chapter_Word_List_Editor_Page):
        return is_word_in_used_word_list__in_32e(c, text)
    else:
        assert False, f"Not support page"


def is_word_in_used_word_list__in_32e(c, text: str):
    text = BehaveUtil.clear_string(text)
    container = w__get_element_by_shown_dom_id(c, [A.InfoRow, A.List])
    assert container is not None, "Can NOT find container"
    row_spans = container.find_elements(By.XPATH, "./div/main/main/span")
    assert row_spans is not None, "Can NOT find row_spans"
    for r in row_spans:
        row_text = r.get_attribute('innerHTML').strip()
        print("row_text = ", row_text)
        if row_text == text:
            return True
    return False


@then('{word} is in word list')
def then_word_is_in_word_list(c, word: str):
    assert is_word_in_used_word_list(c, word)


def find_used_word_row_by_text(c, text: str):
    elm = w__get_element_by_shown_dom_id(c, [A.InfoRow, A.List])
    assert elm is not None
    row_spans = elm.find_elements(By.XPATH, "./div/main/main/span")
    for r in row_spans:
        row_text = r.get_attribute('innerHTML').strip()
        if row_text == text:
            return r.find_element(By.XPATH, "../../..")
    return None


def find_remove_button_of_used_word_row(c, text: str):
    text = BehaveUtil.clear_string(text)
    row = find_used_word_row_by_text(c, text)
    assert row is not None
    add = row.find_element(By.XPATH, "./main/aside/button[2]")
    return add


@when('{mentor} click on [remove button] of word {word} to remove word from list')
def when_click_on_remove_button_of_word(c, mentor: str, word: str):
    btn = find_remove_button_of_used_word_row(c, word)
    assert btn is not None
    print("click_remove_button_of_word_row -> ", btn)
    click_element(c, btn)


@then('{word} is NOT in word list')
def check_word_not_in_used_list(c, word: str):
    assert not is_word_in_used_word_list(c, word)


@when('"{mentor}" click on [surprise icon] of word "{word}" row in [32E_Chapter-Word-List-Editor-Page]')
def when_click_on_surprise_icon_of_word(c, mentor: str, word: str):
    click_on_word_row_button(c, word, 1)


@when('"{mentor}" click on [remove icon] of word "{word}" row in [32E_Chapter-Word-List-Editor-Page]')
def when_click_on_remove_icon_of_word(c, mentor: str, word: str):
    click_on_word_row_button(c, word, 2)